import java.io.*;
import java.util.*;

public class Freq {

	public static void main(String[] args) {

		String[] freqofs = new String[] {"all", "first", "last"};
		double[][] sd = new double[3][4];
		double[][] ent = new double[3][4];
		try {
			for (int mode = 1; mode <= 4; mode++) {
				for (int f = 0; f < freqofs.length; f++) {
					Scanner sc = new Scanner(new File("Data/Mode" + mode + ".txt"));
					double[][] freq = frequencyTable(mode, freqofs[f], sc);
					sd[f][mode-1] = statisticalDistance(mode, freq);
					ent[f][mode-1] = entropy(mode, freq);
					String csv = frequencyTableCSV(mode, freq);
					csv += String.format("%-7s,%7.4f\n", "sd", sd[f][mode-1]);
					csv += String.format("%-7s,%7.4f", "ent", ent[f][mode-1]);
					BufferedWriter out = new BufferedWriter(new FileWriter("Output/Frequency-Mode" + mode + "-" + freqofs[f] + ".csv"));
					out.write(csv);
					out.close();
				}
			}
			BufferedWriter out = new BufferedWriter(new FileWriter("Output/Frequency-StatisticalDistance.csv"));
			out.write(statisticalDistanceCSV(sd));
			out.close();
			out = new BufferedWriter(new FileWriter("Output/Frequency-Entropy.csv"));
			out.write(entropyCSV(ent));
			out.close();
			out = new BufferedWriter(new FileWriter("Output/Frequency-Entropy-Uniform.csv"));
			out.write(entropyCSV(entropyUniform()));
			out.close();
		} catch (Exception e) {
			e.printStackTrace();
			return;
		}


	}

	public static double[][] frequencyTable(int mode, String freqof, Scanner sc) {

		int width = Common.width(mode);
		int height = Common.height(mode);

		int[][] count = new int[width][height];
		for (int w = 0; w < width; w++) {
			for (int h = 0; h < height; h++) {
				count[w][h] = 0;
			}
		}

		String s;
		int numpoints = 0;
		while (sc.hasNext()) {
			s = sc.nextLine();
			String[] points = s.split(",");
			int istart = 0, iend = 0;
			if (freqof.equals("first")) {
				istart = (Common.startPoint(mode) == null) ? 0 : 1;
				iend = istart + 1;
			} else if (freqof.equals("last")) {
				istart = (Common.endPoint(mode) == null) ? points.length - 1 : points.length - 2;
				iend = istart + 1;
			} else if (freqof.equals("all")) {
				istart = (Common.startPoint(mode) == null) ? 0 : 1;
				iend = (Common.endPoint(mode) == null) ? points.length : points.length - 1;
			}
			for (int i = istart; i < iend; i++) {
				String[] coords = points[i].split("x");
				int w = Integer.parseInt(coords[0]);
				int h = Integer.parseInt(coords[1]);
				count[w][h]++;
				numpoints++;
			}
		}
		double[][] freq = new double[width][height];
		for (int h = 0; h < height; h++) {
			for (int w = 0; w < width; w++) {
				freq[w][h] = (double) count[w][h] / (double) numpoints;
			}
		}
		return freq;

	}

	public static String frequencyTableCSV(int mode, double[][] freq) {
		int width = Common.width(mode);
		int height = Common.height(mode);
		HashSet<String> excludedPoints = Common.excludedPoints(mode);
		StringBuffer table = new StringBuffer();
		for (int h = 0; h < height; h++) {
			for (int w = 0; w < width; w++) {
				if (!isExcluded(w, h, excludedPoints)) {
					table.append(String.format("%7.4f", freq[w][h]));
				} else {
					table.append("       ");
				}
				if (w < width - 1) {
					table.append(",");
				}
			}
			table.append("\n");
		}
		return table.toString();
	}

	public static double statisticalDistance(int mode, double[][] freq) {
		int width = Common.width(mode);
		int height = Common.height(mode);
		int gridsize = width * height;
		if (Common.startPoint(mode) != null) gridsize--;
		if (Common.endPoint(mode) != null) gridsize--;
		HashSet<String> excludedPoints = Common.excludedPoints(mode);
		gridsize -= excludedPoints.size();
		double statisticalDistance = 0.0;
		for (int h = 0; h < height; h++) {
			for (int w = 0; w < width; w++) {
				if (!isExcluded(w, h, excludedPoints)) {
					statisticalDistance += 0.5 * Math.abs(freq[w][h] - 1 / (double) gridsize);
				}
			}
		}
		return statisticalDistance;
	}

	public static String statisticalDistanceCSV(double[][] sd) {

		String[] freqofs = new String[] {"all", "first", "last"};
		StringBuffer sdCSV = new StringBuffer();
		sdCSV.append(String.format("%-7s,", "mode"));
		for (int f = 0; f < freqofs.length; f++) {
			sdCSV.append(String.format("%-7s", freqofs[f]));
			if (f < freqofs.length - 1) {
				sdCSV.append(",");
			}
		}
		sdCSV.append("\n");
		for (int mode = 1; mode <= 4; mode++) {
			sdCSV.append(String.format("%-7s,", mode));
			for (int f = 0; f < freqofs.length; f++) {
				sdCSV.append(String.format("%7.4f", sd[f][mode-1]));
				if (f < freqofs.length - 1) {
					sdCSV.append(",");
				}
			}
			sdCSV.append("\n");
		}
		return sdCSV.toString();
	}

	public static double entropy(int mode, double[][] freq) {
		int width = Common.width(mode);
		int height = Common.height(mode);
		double entropy = 0.0;
		for (int h = 0; h < height; h++) {
			for (int w = 0; w < width; w++) {
				if (freq[w][h] > 0) {
					entropy -= freq[w][h] * Math.log(freq[w][h]) / Math.log(2.0);
				}
			}
		}
		return entropy;
	}

	public static double[][] entropyUniform() {
		double[][] ent = new double[3][4];
		for (int mode = 1; mode <= 4; mode++) {
			// all points
			int count = Common.width(mode) * Common.height(mode);
			if (Common.startPoint(mode) != null) count--;
			if (Common.endPoint(mode) != null) count--;
			count -= Common.excludedPoints(mode).size();
			ent[0][mode-1] = Math.log((double) count) / Math.log(2.0);
			// first freely chosen point
			count = Common.width(mode) * Common.height(mode);
			if (Common.startPoint(mode) != null) count--;
			count -= Common.excludedPoints(mode).size();
			ent[1][mode-1] = Math.log((double) count) / Math.log(2.0);
			// last freely chosen point
			count = Common.width(mode) * Common.height(mode);
			if (Common.endPoint(mode) != null) count--;
			count -= Common.excludedPoints(mode).size();
			ent[2][mode-1] = Math.log((double) count) / Math.log(2.0);
		}
		return ent;
	}

	public static String entropyCSV(double[][] ent) {
		String[] freqofs = new String[] {"all", "first", "last"};
		StringBuffer entCSV = new StringBuffer();
		entCSV.append(String.format("%-7s,", "mode"));
		for (int f = 0; f < freqofs.length; f++) {
			entCSV.append(String.format("%-7s", freqofs[f]));
			if (f < freqofs.length - 1) {
				entCSV.append(",");
			}
		}
		entCSV.append("\n");
		for (int mode = 1; mode <= 4; mode++) {
			entCSV.append(String.format("%-7s,", mode));
			for (int f = 0; f < freqofs.length; f++) {
				entCSV.append(String.format("%7.4f", ent[f][mode-1]));
				if (f < freqofs.length - 1) {
					entCSV.append(",");
				}
			}
			entCSV.append("\n");
		}
		return entCSV.toString();
	}

	static boolean isExcluded(int w, int h, HashSet<String> excludedPoints) {
		return excludedPoints.contains(w + "x" + h);
	}

}
