import java.io.*;
import java.util.*;

public class StrokeFreq {


	public static void main(String[] args) {

		if (args.length != 1) {
			System.err.println("Usage: java StrokeFreq mode < file.txt");
			return;
		}
		int mode = Integer.parseInt(args[0]);
		int width = Common.width(mode);
		int height = Common.height(mode);
		
		Scanner sc = new Scanner(System.in);
		HashMap<String,Integer> strokes = new HashMap<String,Integer>();
		while (sc.hasNext()) {
			String pattern = sc.nextLine();
			processStrokes(pattern, strokes);
		}
		int strokeCount = 0;
		for (int strokeNum : strokes.values()) {
			strokeCount += strokeNum;
		}
		HashMap<String,Double> strokesFreq = new HashMap<String,Double>();
		for (String stroke : strokes.keySet()) {
			strokesFreq.put(stroke, (double) strokes.get(stroke) / (double) strokeCount);
		}
		for (int i = -(Common.height(mode)-1); i <= (Common.height(mode)-1); i++) {
			for (int j = -(Common.width(mode)-1); j <= (Common.width(mode)-1); j++) {
				String stroke = i + "S" + j;
				if (strokesFreq.containsKey(stroke)) {
					System.out.printf("%7.4f", strokesFreq.get(stroke));
				} else {
					System.out.printf("%7.4f", 0.0);
				}
				if (j < (Common.width(mode)-1)) System.out.print(",");
			}
			System.out.println();
		}
		double entropy = 0.0;
		for (double strokeFreq : strokesFreq.values()) {
			entropy -= strokeFreq * Math.log(strokeFreq) / Math.log(2.0);
		}
		System.out.printf("%7s,%7.4f\n", "en", entropy);
	}

	public static void processStrokes(String pattern, HashMap<String,Integer> strokes) {
		String[] points = pattern.split(",");
		for (int i = 1; i < points.length; i++) {
			String[] prevPointCoords = points[i-1].split("x");
			int prevW = Integer.parseInt(prevPointCoords[0]);
			int prevH = Integer.parseInt(prevPointCoords[1]);
			String[] currPointCoords = points[i].split("x");
			int currW = Integer.parseInt(currPointCoords[0]);
			int currH = Integer.parseInt(currPointCoords[1]);
			String stroke = (currW - prevW) + "S" + (currH - prevH);
			if (strokes.containsKey(stroke)) {
				strokes.put(stroke, strokes.get(stroke) + 1);
			} else {
				strokes.put(stroke, 1);
			}
		}
	}

}


