import java.io.*;
import java.util.*;

public class Sym {


	public static void main(String[] args) {

		if (args.length != 1) {
			System.err.println("Usage: java Sym mode < file.txt");
			return;
		}
		int mode = Integer.parseInt(args[0]);
		int width = Common.width(mode);
		int height = Common.height(mode);
		
		Scanner sc = new Scanner(System.in);
		int count = 0;
		double vpsym = 0.0;
		double hpsym = 0.0;
		while (sc.hasNext()) {
			String pattern = sc.nextLine();
			boolean[][] grid = patternToGrid(width, height, pattern);
			count++;
			vpsym += bestVerticalPointSymmetry(width, height, grid);
			hpsym += bestHorizontalPointSymmetry(width, height, grid);
		}
		System.out.printf("Average vertical point symmetry: %6.4f\n", vpsym / (double) count);
		System.out.printf("Average horizontal point symmetry: %6.4f\n", hpsym / (double) count);
	}

	public static boolean[][] patternToGrid(int width, int height, String pattern) {
		boolean[][] grid = new boolean[width][height];
		for (int w = 0; w < width; w++) {
			for (int h = 0; h < height; h++) {
				grid[w][h] = false;
			}
		}
		String[] points = pattern.split(",");
		for (int i = 0; i < points.length; i++) {
			String[] coords = points[i].split("x");
			int w = Integer.parseInt(coords[0]);
			int h = Integer.parseInt(coords[1]);
			grid[w][h] = true;
		}
		return grid;
	}

	public static int pointsUsed(int width, int height, boolean[][] grid) {
		int used = 0;
		for (int w = 0; w < width; w++) {
			for (int h = 0; h < height; h++) {
				if (grid[w][h]) {
					used++;
				}
			}
		}
		return used;
	}

	public static double bestVerticalPointSymmetry(int width, int height, boolean[][] grid) {
		double bestSoFar = 0.0;
		for (int axis = 0; axis < width; axis++) {
			bestSoFar = Math.max(bestSoFar, verticalPointSymmetryAxisOn(width, height, axis, grid));
		}
		for (int axis = 0; axis < width - 1; axis++) {
			bestSoFar = Math.max(bestSoFar, verticalPointSymmetryAxisBetween(width, height, axis, grid));
		}
		return bestSoFar;
	}

	public static double verticalPointSymmetryAxisOn(int width, int height, int axis, boolean[][] grid) {
		int totalUsed = pointsUsed(width, height, grid);
		int matchingUsed = 0;
		int matchableWidth = Math.min(axis, width-axis-1);
		for (int h = 0; h < height; h++) {
			// points on the axis
			if (grid[axis][h]) {
				matchingUsed++;
			}
			// points equidistant from axis
			for (int woff = 1; woff <= matchableWidth; woff++) {
				if (grid[axis+woff][h] && grid[axis-woff][h]) {
					matchingUsed += 2;
				}
			}
		}
		return (double) matchingUsed / (double) totalUsed;
	}

	public static double verticalPointSymmetryAxisBetween(int width, int height, int axisAfter, boolean[][] grid) {
		int totalUsed = pointsUsed(width, height, grid);
		int matchingUsed = 0;
		int matchableWidth = Math.min(axisAfter+1, width-1-axisAfter);
		for (int h = 0; h < height; h++) {
			// points equidistant from axis
			for (int woff = 0; woff < matchableWidth; woff++) {
				if (grid[axisAfter+1+woff][h] && grid[axisAfter-woff][h]) {
					matchingUsed += 2;
				}
			}
		}
		return (double) matchingUsed / (double) totalUsed;
	}

	public static double bestHorizontalPointSymmetry(int width, int height, boolean[][] grid) {
		double bestSoFar = 0.0;
		for (int axis = 0; axis < height; axis++) {
			bestSoFar = Math.max(bestSoFar, horizontalPointSymmetryAxisOn(width, height, axis, grid));
		}
		for (int axis = 0; axis < height - 1; axis++) {
			bestSoFar = Math.max(bestSoFar, horizontalPointSymmetryAxisBetween(width, height, axis, grid));
		}
		return bestSoFar;
	}

	public static boolean[][] gridTransposed(int width, int height, boolean[][] grid) {
		boolean[][] gridTransposed = new boolean[height][width];
		for (int w = 0; w < width; w++) {
			for (int h = 0; h < height; h++) {
				gridTransposed[h][w] = grid[w][h];
			}
		}
		return gridTransposed;
	}

	public static double horizontalPointSymmetryAxisOn(int width, int height, int axis, boolean[][] grid) {
		return verticalPointSymmetryAxisOn(height, width, axis, gridTransposed(width, height, grid));
	}

	public static double horizontalPointSymmetryAxisBetween(int width, int height, int axisAfter, boolean[][] grid) {
		return verticalPointSymmetryAxisBetween(height, width, axisAfter, gridTransposed(width, height, grid));
	}

}

